@@ -7,31 +7,71 @@ variety of line styles, markers and colors.
 from __future__ import (absolute_import, division, print_function,
                         unicode_literals)
 
-from matplotlib.externals import six
+import six
 
 import warnings
 
 import numpy as np
-from numpy import ma
-from matplotlib import verbose
-from . import artist
-from .artist import Artist
-from .cbook import (iterable, is_string_like, is_numlike, ls_mapper_r,
-                    pts_to_prestep, pts_to_poststep, pts_to_midstep)
 
-from .colors import colorConverter
+from . import artist, colors as mcolors, docstring, rcParams
+from .artist import Artist, allow_rasterization
+from .cbook import (
+    _to_unmasked_float_array, iterable, is_numlike, ls_mapper, ls_mapper_r,
+    STEP_LOOKUP_MAP)
+from .markers import MarkerStyle
 from .path import Path
 from .transforms import Bbox, TransformedPath, IdentityTransform
 
-from matplotlib import rcParams
-from .artist import allow_rasterization
-from matplotlib import docstring
-from matplotlib.markers import MarkerStyle
 # Imported here for backward compatibility, even though they don't
 # really belong.
-from matplotlib.markers import TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN
-from matplotlib.markers import CARETLEFT, CARETRIGHT, CARETUP, CARETDOWN
-from matplotlib import _path
+from numpy import ma
+from . import _path
+from .markers import (
+    CARETLEFT, CARETRIGHT, CARETUP, CARETDOWN,
+    CARETLEFTBASE, CARETRIGHTBASE, CARETUPBASE, CARETDOWNBASE,
+    TICKLEFT, TICKRIGHT, TICKUP, TICKDOWN)
+
+
+def _get_dash_pattern(style):
+    """Convert linestyle -> dash pattern
+    """
+    # go from short hand -> full strings
+    if isinstance(style, six.string_types):
+        style = ls_mapper.get(style, style)
+    # un-dashed styles
+    if style in ['solid', 'None']:
+        offset, dashes = None, None
+    # dashed styles
+    elif style in ['dashed', 'dashdot', 'dotted']:
+        offset = 0
+        dashes = tuple(rcParams['lines.{}_pattern'.format(style)])
+    #
+    elif isinstance(style, tuple):
+        offset, dashes = style
+    else:
+        raise ValueError('Unrecognized linestyle: %s' % str(style))
+
+    # normalize offset to be positive and shorter than the dash cycle
+    if dashes is not None and offset is not None:
+        dsum = sum(dashes)
+        if dsum:
+            offset %= dsum
+
+    return offset, dashes
+
+
+def _scale_dashes(offset, dashes, lw):
+    if not rcParams['lines.scale_dashes']:
+        return offset, dashes
+
+    scaled_offset = scaled_dashes = None
+    if offset is not None:
+        scaled_offset = offset * lw
+    if dashes is not None:
+        scaled_dashes = [x * lw if x is not None else None
+                         for x in dashes]
+
+    return scaled_offset, scaled_dashes
 
 
 def segment_hits(cx, cy, x, y, radius):
@@ -102,6 +142,9 @@ def _mark_every_path(markevery, tpath, affine, ax_transform):
     # if just an int, assume starting at 0 and make a tuple
     elif isinstance(markevery, int):
         markevery = (0, markevery)
+    # if just an numpy int, assume starting at 0 and make a tuple
+    elif isinstance(markevery, np.integer):
+        markevery = (0, markevery.item())
 
     if isinstance(markevery, tuple):
         if len(markevery) != 2:
@@ -218,12 +261,12 @@ class Line2D(Artist):
         'steps': '_draw_steps_pre',
     }
 
+    # drawStyles should now be deprecated.
     drawStyles = {}
     drawStyles.update(_drawStyles_l)
     drawStyles.update(_drawStyles_s)
     # Need a list ordered with long names first:
-    drawStyleKeys = (list(six.iterkeys(_drawStyles_l)) +
-                     list(six.iterkeys(_drawStyles_s)))
+    drawStyleKeys = list(_drawStyles_l) + list(_drawStyles_s)
 
     # Referenced here to maintain API.  These are defined in
     # MarkerStyle
@@ -314,6 +357,18 @@ class Line2D(Artist):
         if solid_joinstyle is None:
             solid_joinstyle = rcParams['lines.solid_joinstyle']
 
+        if isinstance(linestyle, six.string_types):
+            ds, ls = self._split_drawstyle_linestyle(linestyle)
+            if ds is not None and drawstyle is not None and ds != drawstyle:
+                raise ValueError("Inconsistent drawstyle ({0!r}) and "
+                                 "linestyle ({1!r})".format(drawstyle,
+                                                            linestyle)
+                                 )
+            linestyle = ls
+
+            if ds is not None:
+                drawstyle = ds
+
         if drawstyle is None:
             drawstyle = 'default'
 
@@ -328,15 +383,23 @@ class Line2D(Artist):
 
         self._linestyles = None
         self._drawstyle = None
-        self._linewidth = None
+        self._linewidth = linewidth
+
+        # scaled dash + offset
+        self._dashSeq = None
+        self._dashOffset = 0
+        # unscaled dash + offset
+        # this is needed scaling the dash pattern by linewidth
+        self._us_dashSeq = None
+        self._us_dashOffset = 0
+
         self.set_linestyle(linestyle)
         self.set_drawstyle(drawstyle)
         self.set_linewidth(linewidth)
 
         self._color = None
         self.set_color(color)
-        self._marker = MarkerStyle()
-        self.set_marker(marker)
+        self._marker = MarkerStyle(marker, fillstyle)
 
         self._markevery = None
         self._markersize = None
@@ -346,8 +409,6 @@ class Line2D(Artist):
         self.set_antialiased(antialiased)
         self.set_markersize(markersize)
 
-        self._dashSeq = None
-
         self._markeredgecolor = None
         self._markeredgewidth = None
         self._markerfacecolor = None
@@ -358,8 +419,6 @@ class Line2D(Artist):
         self.set_markeredgecolor(markeredgecolor)
         self.set_markeredgewidth(markeredgewidth)
 
-        self.set_fillstyle(fillstyle)
-
         self.verticalOffset = None
 
         # update kwargs before updating data to give the caller a
@@ -384,12 +443,6 @@ class Line2D(Artist):
 
         self.set_data(xdata, ydata)
 
-    def __getstate__(self):
-        state = super(Line2D, self).__getstate__()
-        # _linefunc will be restored on draw time.
-        state.pop('_lineFunc', None)
-        return state
-
     def contains(self, mouseevent):
         """
         Test whether the mouse event occurred on the line.  The pick
@@ -405,7 +458,7 @@ class Line2D(Artist):
 
         TODO: sort returned indices by distance
         """
-        if six.callable(self._contains):
+        if callable(self._contains):
             return self._contains(self, mouseevent)
 
         if not is_numlike(self.pickradius):
@@ -437,8 +490,7 @@ class Line2D(Artist):
         # application has set the error flags such that an exception is raised
         # on overflow, we temporarily set the appropriate error flags here and
         # set them back when we are finished.
-        olderrflags = np.seterr(all='ignore')
-        try:
+        with np.errstate(all='ignore'):
             # Check for collision
             if self._linestyle in ['None', None]:
                 # If no line, return the nearby point(s)
@@ -447,20 +499,11 @@ class Line2D(Artist):
             else:
                 # If line, return the nearby segment(s)
                 ind = segment_hits(mouseevent.x, mouseevent.y, xt, yt, pixels)
-        finally:
-            np.seterr(**olderrflags)
+                if self._drawstyle.startswith("steps"):
+                    ind //= 2
 
         ind += self.ind_offset
 
-        # Debugging message
-        if False and self._label != '':
-            print("Checking line", self._label,
-                  "at", mouseevent.x, mouseevent.y)
-            print('xt', xt)
-            print('yt', yt)
-            #print 'dx,dy', (xt-mouseevent.x)**2., (yt-mouseevent.y)**2.
-            print('ind', ind)
-
         # Return the point(s) within radius
         return len(ind) > 0, dict(ind=ind)
 
@@ -469,9 +512,15 @@ class Line2D(Artist):
         return self.pickradius
 
     def set_pickradius(self, d):
-        """Sets the pick radius used for containment tests
+        """Set the pick radius used for containment tests.
 
-        ACCEPTS: float distance in points
+        ..
+            ACCEPTS: float distance in points
+
+        Parameters
+        ----------
+        d : float
+            Pick radius, in points.
         """
         self.pickradius = d
 
@@ -501,8 +550,8 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        every: None | int | length-2 tuple of int | slice | list/array of int |
-        float | length-2 tuple of float
+        every: None | int | length-2 tuple of int | slice | list/array of int \
+| float | length-2 tuple of float
             Which markers to plot.
 
             - every=None, every point will be plotted.
@@ -556,7 +605,7 @@ class Line2D(Artist):
         ACCEPTS: float distance in points or callable pick function
         ``fn(artist, event)``
         """
-        if six.callable(p):
+        if callable(p):
             self._contains = p
         else:
             self.pickradius = p
@@ -606,37 +655,17 @@ class Line2D(Artist):
     def recache(self, always=False):
         if always or self._invalidx:
             xconv = self.convert_xunits(self._xorig)
-            if ma.isMaskedArray(self._xorig):
-                x = ma.asarray(xconv, np.float_).filled(np.nan)
-            else:
-                x = np.asarray(xconv, np.float_)
-            x = x.ravel()
+            x = _to_unmasked_float_array(xconv).ravel()
         else:
             x = self._x
         if always or self._invalidy:
             yconv = self.convert_yunits(self._yorig)
-            if ma.isMaskedArray(self._yorig):
-                y = ma.asarray(yconv, np.float_).filled(np.nan)
-            else:
-                y = np.asarray(yconv, np.float_)
-            y = y.ravel()
+            y = _to_unmasked_float_array(yconv).ravel()
         else:
             y = self._y
 
-        if len(x) == 1 and len(y) > 1:
-            x = x * np.ones(y.shape, np.float_)
-        if len(y) == 1 and len(x) > 1:
-            y = y * np.ones(x.shape, np.float_)
-
-        if len(x) != len(y):
-            raise RuntimeError('xdata and ydata must be the same length')
-
-        self._xy = np.empty((len(x), 2), dtype=np.float_)
-        self._xy[:, 0] = x
-        self._xy[:, 1] = y
-
-        self._x = self._xy[:, 0]  # just a view
-        self._y = self._xy[:, 1]  # just a view
+        self._xy = np.column_stack(np.broadcast_arrays(x, y)).astype(float)
+        self._x, self._y = self._xy.T  # views
 
         self._subslice = False
         if (self.axes and len(x) > 1000 and self._is_sorted(x) and
@@ -658,7 +687,9 @@ class Line2D(Artist):
             interpolation_steps = self._path._interpolation_steps
         else:
             interpolation_steps = 1
-        self._path = Path(self._xy, None, interpolation_steps)
+        xy = STEP_LOOKUP_MAP[self._drawstyle](*self._xy.T)
+        self._path = Path(np.asarray(xy).T,
+                          _interpolation_steps=interpolation_steps)
         self._transformed_path = None
         self._invalidx = False
         self._invalidy = False
@@ -671,8 +702,9 @@ class Line2D(Artist):
         """
         # Masked arrays are now handled by the Path class itself
         if subslice is not None:
-            _steps = self._path._interpolation_steps
-            _path = Path(self._xy[subslice, :], _interpolation_steps=_steps)
+            xy = STEP_LOOKUP_MAP[self._drawstyle](*self._xy[subslice, :].T)
+            _path = Path(np.asarray(xy).T,
+                         _interpolation_steps=self._path._interpolation_steps)
         else:
             _path = self._path
         self._transformed_path = TransformedPath(_path, self.get_transform())
@@ -726,13 +758,9 @@ class Line2D(Artist):
             renderer = PathEffectRenderer(self.get_path_effects(), renderer)
 
         renderer.open_group('line2d', self.get_gid())
-        funcname = self._lineStyles.get(self._linestyle, '_draw_nothing')
-        if funcname != '_draw_nothing':
+        if self._lineStyles[self._linestyle] != '_draw_nothing':
             tpath, affine = transf_path.get_transformed_path_and_affine()
             if len(tpath.vertices):
-                self._lineFunc = getattr(self, funcname)
-                funcname = self.drawStyles.get(self._drawstyle, '_draw_lines')
-                drawFunc = getattr(self, funcname)
                 gc = renderer.new_gc()
                 self._set_gc_clip(gc)
 
@@ -755,23 +783,25 @@ class Line2D(Artist):
                 if self.get_sketch_params() is not None:
                     gc.set_sketch_params(*self.get_sketch_params())
 
-                drawFunc(renderer, gc, tpath, affine.frozen())
+                gc.set_dashes(self._dashOffset, self._dashSeq)
+                renderer.draw_path(gc, tpath, affine.frozen())
                 gc.restore()
 
-        if self._marker:
+        if self._marker and self._markersize > 0:
             gc = renderer.new_gc()
             self._set_gc_clip(gc)
             rgbaFace = self._get_rgba_face()
             rgbaFaceAlt = self._get_rgba_face(alt=True)
             edgecolor = self.get_markeredgecolor()
-            if is_string_like(edgecolor) and edgecolor.lower() == 'none':
+            if (isinstance(edgecolor, six.string_types)
+                    and edgecolor.lower() == 'none'):
                 gc.set_linewidth(0)
                 gc.set_foreground(rgbaFace, isRGBA=True)
             else:
                 gc.set_foreground(edgecolor)
                 gc.set_linewidth(self._markeredgewidth)
                 mec = self._markeredgecolor
-                if (is_string_like(mec) and mec == 'auto' and
+                if (isinstance(mec, six.string_types) and mec == 'auto' and
                         rgbaFace is not None):
                     gc.set_alpha(rgbaFace[3])
                 else:
@@ -797,11 +827,13 @@ class Line2D(Artist):
                 marker_path = marker.get_path()
                 marker_trans = marker.get_transform()
                 w = renderer.points_to_pixels(self._markersize)
-                if marker.get_marker() != ',':
+
+                if (isinstance(marker.get_marker(), six.string_types) and
+                        marker.get_marker() == ','):
+                    gc.set_linewidth(0)
+                else:
                     # Don't scale for pixels, and don't stroke them
                     marker_trans = marker_trans.scale(w)
-                else:
-                    gc.set_linewidth(0)
 
                 renderer.draw_markers(gc, marker_path, marker_trans,
                                       subsampled, affine.frozen(),
@@ -811,7 +843,7 @@ class Line2D(Artist):
                 if alt_marker_path:
                     alt_marker_trans = marker.get_alt_transform()
                     alt_marker_trans = alt_marker_trans.scale(w)
-                    if (is_string_like(mec) and mec == 'auto' and
+                    if (isinstance(mec, six.string_types) and mec == 'auto' and
                             rgbaFaceAlt is not None):
                         gc.set_alpha(rgbaFaceAlt[3])
                     else:
@@ -846,13 +878,13 @@ class Line2D(Artist):
 
     def get_markeredgecolor(self):
         mec = self._markeredgecolor
-        if (is_string_like(mec) and mec == 'auto'):
-            if self._marker.get_marker() in ('.', ','):
-                return self._color
-            if self._marker.is_filled() and self.get_fillstyle() != 'none':
-                return 'k'  # Bad hard-wired default...
-            else:
-                return self._color
+        if isinstance(mec, six.string_types) and mec == 'auto':
+            if rcParams['_internal.classic_mode']:
+                if self._marker.get_marker() in ('.', ','):
+                    return self._color
+                if self._marker.is_filled() and self.get_fillstyle() != 'none':
+                     return 'k'  # Bad hard-wired default...
+            return self._color
         else:
             return mec
 
@@ -865,7 +897,7 @@ class Line2D(Artist):
         else:
             fc = self._markerfacecolor
 
-        if (is_string_like(fc) and fc.lower() == 'auto'):
+        if (isinstance(fc, six.string_types) and fc.lower() == 'auto'):
             if self.get_fillstyle() == 'none':
                 return 'none'
             else:
@@ -963,6 +995,10 @@ class Line2D(Artist):
         ACCEPTS: ['default' | 'steps' | 'steps-pre' | 'steps-mid' |
                   'steps-post']
         """
+        if drawstyle is None:
+            drawstyle = 'default'
+        if drawstyle not in self.drawStyles:
+            raise ValueError('Unrecognized drawstyle {!r}'.format(drawstyle))
         if self._drawstyle != drawstyle:
             self.stale = True
         self._drawstyle = drawstyle
@@ -974,9 +1010,45 @@ class Line2D(Artist):
         ACCEPTS: float value in points
         """
         w = float(w)
+
         if self._linewidth != w:
             self.stale = True
         self._linewidth = w
+        # rescale the dashes + offset
+        self._dashOffset, self._dashSeq = _scale_dashes(
+            self._us_dashOffset, self._us_dashSeq, self._linewidth)
+
+    def _split_drawstyle_linestyle(self, ls):
+        '''Split drawstyle from linestyle string
+
+        If `ls` is only a drawstyle default to returning a linestyle
+        of '-'.
+
+        Parameters
+        ----------
+        ls : str
+            The linestyle to be processed
+
+        Returns
+        -------
+        ret_ds : str or None
+            If the linestyle string does not contain a drawstyle prefix
+            return None, otherwise return it.
+
+        ls : str
+            The linestyle with the drawstyle (if any) stripped.
+        '''
+        ret_ds = None
+        for ds in self.drawStyleKeys:  # long names are first in the list
+            if ls.startswith(ds):
+                ret_ds = ds
+                if len(ls) > len(ds):
+                    ls = ls[len(ds):]
+                else:
+                    ls = '-'
+                break
+
+        return ret_ds, ls
 
     def set_linestyle(self, ls):
         """
@@ -989,7 +1061,7 @@ class Line2D(Artist):
         ===========================   =================
         ``'-'`` or ``'solid'``        solid line
         ``'--'`` or  ``'dashed'``     dashed line
-        ``'-.'`` or  ``'dash_dot'``   dash-dotted line
+        ``'-.'`` or  ``'dashdot'``    dash-dotted line
         ``':'`` or ``'dotted'``       dotted line
         ``'None'``                    draw nothing
         ``' '``                       draw nothing
@@ -1019,39 +1091,34 @@ class Line2D(Artist):
 
         Parameters
         ----------
-        ls : { '-',  '--', '-.', ':'} and more see description
+        ls : { ``'-'``,  ``'--'``, ``'-.'``, ``':'``} and more see description
             The line style.
         """
-        if not is_string_like(ls):
-            if len(ls) != 2:
-                raise ValueError()
-
-            self.set_dashes(ls[1])
-            self._linestyle = "--"
-            return
-
-        for ds in self.drawStyleKeys:  # long names are first in the list
-            if ls.startswith(ds):
+        if isinstance(ls, six.string_types):
+            ds, ls = self._split_drawstyle_linestyle(ls)
+            if ds is not None:
                 self.set_drawstyle(ds)
-                if len(ls) > len(ds):
-                    ls = ls[len(ds):]
-                else:
-                    ls = '-'
-                break
 
-        if ls in [' ', '', 'none']:
-            ls = 'None'
+            if ls in [' ', '', 'none']:
+                ls = 'None'
 
-        if ls not in self._lineStyles:
-            try:
-                ls = ls_mapper_r[ls]
-            except KeyError:
-                raise ValueError(("You passed in an invalid linestyle, "
-                                  "`{}`.  See "
-                                  "docs of Line2D.set_linestyle for "
-                                  "valid values.").format(ls))
+            if ls not in self._lineStyles:
+                try:
+                    ls = ls_mapper_r[ls]
+                except KeyError:
+                    raise ValueError(("You passed in an invalid linestyle, "
+                                      "`{0}`.  See "
+                                      "docs of Line2D.set_linestyle for "
+                                      "valid values.").format(ls))
+            self._linestyle = ls
+        else:
+            self._linestyle = '--'
 
-        self._linestyle = ls
+        # get the unscaled dashes
+        self._us_dashOffset, self._us_dashSeq = _get_dash_pattern(ls)
+        # compute the linewidth scaled dashes
+        self._dashOffset, self._dashSeq = _scale_dashes(
+            self._us_dashOffset, self._us_dashSeq, self._linewidth)
 
     @docstring.dedent_interpd
     def set_marker(self, marker):
@@ -1061,7 +1128,7 @@ class Line2D(Artist):
         ACCEPTS: :mod:`A valid marker style <matplotlib.markers>`
 
         Parameters
-        -----------
+        ----------
 
         marker: marker style
             See `~matplotlib.markers` for full description of possible
@@ -1079,7 +1146,8 @@ class Line2D(Artist):
         """
         if ec is None:
             ec = 'auto'
-        if self._markeredgecolor != ec:
+        if self._markeredgecolor is None or \
+           np.any(self._markeredgecolor != ec):
             self.stale = True
         self._markeredgecolor = ec
 
@@ -1103,7 +1171,7 @@ class Line2D(Artist):
         """
         if fc is None:
             fc = 'auto'
-        if self._markerfacecolor != fc:
+        if np.any(self._markerfacecolor != fc):
             self.stale = True
         self._markerfacecolor = fc
 
@@ -1115,7 +1183,7 @@ class Line2D(Artist):
         """
         if fc is None:
             fc = 'auto'
-        if self._markerfacecoloralt != fc:
+        if np.any(self._markerfacecoloralt != fc):
             self.stale = True
         self._markerfacecoloralt = fc
 
@@ -1161,53 +1229,7 @@ class Line2D(Artist):
         if seq == (None, None) or len(seq) == 0:
             self.set_linestyle('-')
         else:
-            self.set_linestyle('--')
-        if self._dashSeq != seq:
-            self.stale = True
-        self._dashSeq = seq  # TODO: offset ignored for now
-
-    def _draw_lines(self, renderer, gc, path, trans):
-        self._lineFunc(renderer, gc, path, trans)
-
-    def _draw_steps_pre(self, renderer, gc, path, trans):
-        steps = np.vstack(pts_to_prestep(*self._xy.T)).T
-
-        path = Path(steps)
-        path = path.transformed(self.get_transform())
-        self._lineFunc(renderer, gc, path, IdentityTransform())
-
-    def _draw_steps_post(self, renderer, gc, path, trans):
-        steps = np.vstack(pts_to_poststep(*self._xy.T)).T
-
-        path = Path(steps)
-        path = path.transformed(self.get_transform())
-        self._lineFunc(renderer, gc, path, IdentityTransform())
-
-    def _draw_steps_mid(self, renderer, gc, path, trans):
-        steps = np.vstack(pts_to_midstep(*self._xy.T)).T
-
-        path = Path(steps)
-        path = path.transformed(self.get_transform())
-        self._lineFunc(renderer, gc, path, IdentityTransform())
-
-    def _draw_solid(self, renderer, gc, path, trans):
-        gc.set_linestyle('solid')
-        renderer.draw_path(gc, path, trans)
-
-    def _draw_dashed(self, renderer, gc, path, trans):
-        gc.set_linestyle('dashed')
-        if self._dashSeq is not None:
-            gc.set_dashes(0, self._dashSeq)
-
-        renderer.draw_path(gc, path, trans)
-
-    def _draw_dash_dot(self, renderer, gc, path, trans):
-        gc.set_linestyle('dashdot')
-        renderer.draw_path(gc, path, trans)
-
-    def _draw_dotted(self, renderer, gc, path, trans):
-        gc.set_linestyle('dotted')
-        renderer.draw_path(gc, path, trans)
+            self.set_linestyle((0, seq))
 
     def update_from(self, other):
         """copy properties from other to self"""
@@ -1221,6 +1243,9 @@ class Line2D(Artist):
         self._markeredgecolor = other._markeredgecolor
         self._markeredgewidth = other._markeredgewidth
         self._dashSeq = other._dashSeq
+        self._us_dashSeq = other._us_dashSeq
+        self._dashOffset = other._dashOffset
+        self._us_dashOffset = other._us_dashOffset
         self._dashcapstyle = other._dashcapstyle
         self._dashjoinstyle = other._dashjoinstyle
         self._solidcapstyle = other._solidcapstyle
@@ -1231,24 +1256,17 @@ class Line2D(Artist):
                                    other._marker.get_fillstyle())
         self._drawstyle = other._drawstyle
 
-    def _get_rgb_face(self, alt=False):
-        facecolor = self._get_markerfacecolor(alt=alt)
-        if is_string_like(facecolor) and facecolor.lower() == 'none':
-            rgbFace = None
-        else:
-            rgbFace = colorConverter.to_rgb(facecolor)
-        return rgbFace
-
     def _get_rgba_face(self, alt=False):
         facecolor = self._get_markerfacecolor(alt=alt)
-        if is_string_like(facecolor) and facecolor.lower() == 'none':
+        if (isinstance(facecolor, six.string_types)
+                and facecolor.lower() == 'none'):
             rgbaFace = None
         else:
-            rgbaFace = colorConverter.to_rgba(facecolor, self._alpha)
+            rgbaFace = mcolors.to_rgba(facecolor, self._alpha)
         return rgbaFace
 
     def _get_rgba_ln_color(self, alt=False):
-        return colorConverter.to_rgba(self._color, self._alpha)
+        return mcolors.to_rgba(self._color, self._alpha)
 
     # some aliases....
     def set_aa(self, val):
@@ -1474,15 +1492,8 @@ class VertexSelector(object):
         """When the line is picked, update the set of selected indicies."""
         if event.artist is not self.line:
             return
-
-        for i in event.ind:
-            if i in self.ind:
-                self.ind.remove(i)
-            else:
-                self.ind.add(i)
-
-        ind = list(self.ind)
-        ind.sort()
+        self.ind ^= set(event.ind)
+        ind = sorted(self.ind)
         xdata, ydata = self.line.get_data()
         self.process_selected(ind, xdata[ind], ydata[ind])
 
